#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <sys/stat.h>

struct patch_info
{
	const char * name;
	const char * old;
	const char * new;
	size_t length;
	off_t offset;
	int skip;
};

#if DROMED==1
# include "d1v132.c"
#endif
#if DROMED==2
# include "d2v118.c"
#endif
#if DROMED==3
# include "s2v212.c"
#endif

#define MAX(a,b) ( (a) > (b) ? (a) : (b) )

static void display(const char* msg)
{
	fputs(msg, stderr);
	fputs("Press Enter...", stderr);
	fflush(stderr);
	getchar();
}

static void show(const char* msg1, const char* msg2)
{
	fputs(msg1, stderr);
	fputs(msg2, stderr);
	fflush(stderr);
}

int main(int argc, char *argv[])
{
	FILE* fh;
	char* name;
	struct stat sb;
	char* buf;
	size_t nbytes;
	struct patch_info* patch;

	init();

	fputs("Total Patch for ", stdout);
	fputs(original_name, stdout);
	fputs("\nPatches available:", stdout);
	for (patch = Patches; patch->name; patch++)
	{
		fputs(" [", stdout); fputs(patch->name, stdout); fputs("]", stdout);
	}
	fputs("\n", stdout);
	fflush(stdout);
	if (argc < 2)
	{
		name = alloca(FILENAME_MAX);
		fputs("Enter path to DromEd.exe...\n>", stdout);
		fflush(stdout);
		fgets(name, FILENAME_MAX, stdin);
		name[strlen(name)-1] = '\0';
	}
	else
		name = argv[1];

	if (stat(name, &sb) < 0)
	{
		display("Could not open file.\n");
		return 1;
	}
	if ((unsigned)sb.st_size != original_size)
	{
		display("Could not verify file.\n");
		return 0;
	}

	nbytes = 0;
	for (patch = Patches; patch->name; patch++)
		nbytes = MAX(nbytes, patch->length);

	buf = malloc(nbytes);
	if (buf == NULL)
	{
		display("Memory error.\n");
		return 1;
	}

	fh = fopen(name, "r+b");
	if (fh == NULL)
	{
		display("Could not open file.\n");
		free(buf);
		return 1;
	}

	for (patch = Patches; patch->name; patch++)
	{
		fseek(fh, patch->offset, SEEK_SET);
		nbytes = fread(buf, 1, patch->length, fh);
		if (nbytes != patch->length)
		{
			display("Could not verify file.\n");
			fclose(fh);
			free(buf);
			return 0;
		}
		if (0 == memcmp(buf, patch->new, patch->length))
		{
			show(patch->name, " Patch already applied.\n");
			patch->skip = 1;
		}
		else if (0 != memcmp(buf, patch->old, patch->length))
		{
			display("Could not verify file.\n");
			fclose(fh);
			free(buf);
			return 0;
		}
	}


	for (patch = Patches; patch->name; patch++)
	{
		if (patch->skip)
			continue;
		show(patch->name, ": Applying patch.\n");
		fseek(fh, patch->offset, SEEK_SET);
		nbytes = fwrite(patch->new, 1, patch->length, fh);
		if (nbytes != patch->length)
		{
			display("Write error.\n");
			fclose(fh);
			free(buf);
			return 0;
		}
	}

	fclose(fh);
	free(buf);

	display("All patches applied.\n");
	
	return 0;
}
