/* Macros to prevent isspace from being inlined. */
/* GNU C */
#define __STRICT_ANSI__
/* MSVC */
#define _MT
/* Borland */
#define __USELOCALES__
#include <string.h>
#include <ctype.h>
/* Of the three, GCC makes the most sensible code.
 * MSVC is similar, and was actually slightly smaller.
 * Borland is the most optimized generally,
 * but I couldn't turn off frame pointers for some reason.
 * The clincher came down to what strlen() generated.
 * GCC was the only one to use the built-in "repnz scas" instruction.
 * MSVC used a loop, and Borland wanted to make a function call.
 */

extern char* basename(char* s);
extern char* strip(char* s);

// 64 bytes
char* strip(char* s)
{
  if (s)
  {
    while (isspace(*s)) s++;
    if (*s)
      basename(s);
  }
  return s;
}

// 112 bytes
char* basename(char* s)
{
  register char* p = s;
  register unsigned int c = strlen(s);
  register char* q = s + c - 1;
  while (p <= q && isspace(*q))
  {
    *q-- = 0;
    c--;
  }
  while (p <= q)
  {
    if (*q == '\\' || *q == '/' || *q == ':')
    {
      memmove(p, q+1, c-(q-p));
      break;
    }
    q--;
  }
  return p;
}

#if TEST
int main(int argc, char** argv)
{
	char* s = NULL;
	if (argc > 1)
		s = strdup(argv[1]);
	printf("%s", strip(s));
	free(s);
	return 0;
}
#endif
